package gov.nasa.worldwind.examples;


import com.sun.opengl.util.j2d.TextRenderer;
import gov.nasa.worldwind.Configuration;
import gov.nasa.worldwind.util.BasicDragger;
import gov.nasa.worldwind.event.*;
import gov.nasa.worldwind.geom.*;

import gov.nasa.worldwind.layers.SurfaceShapeLayer;

import gov.nasa.worldwind.pick.PickedObjectList;
import gov.nasa.worldwind.render.*;


import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;

import java.io.*;
import java.lang.*;
import java.awt.*;
import java.util.Timer;
import java.util.TimerTask;

public class Sensor
{
protected static class AppFrame extends JFrame
{
		private Dimension canvasSize = new Dimension(1000, 800);
		private Dimension selectPanelSize = new Dimension(200, 100);
		private ApplicationTemplate.AppPanel wwjPanel;
		private SurfaceShapeLayer layer1 = new SurfaceShapeLayer();
		private SurfaceShapeLayer layer2 = new SurfaceShapeLayer();
		private TextRenderer textRenderer = new TextRenderer(java.awt.Font.decode("Arial-Plain-13"), true, false);

		double angle;
		double radarHole, minorRadius, majorRadius, radarRim;

		Position temp1 = null;
		Position temp2 = null;



		//Initialize the radar footprint (circle)
		LatLon position = new LatLon(Angle.fromDegrees(0), Angle.fromDegrees(0));

		Renderable[] hits = new Renderable[20];

		//SurfaceSensor(Latlong position, holeRadius, outerRadius, Angle heading, int intervals, Angle FOV)
		Renderable radar = new SurfaceSensor(position, 20e3, 100e3, Angle.fromDegrees(0),100,Angle.fromDegrees(200));
		Renderable EO = new SurfaceSensor(position, 15e3, 25e3, Angle.fromDegrees(270),100,Angle.fromDegrees(30));

		Color color, Bcolor;
		float alpha;
		AbstractSurfaceShape rdrShape;
		AbstractSurfaceShape EOShape;

		public AppFrame()
		{
			// Create the WorldWindow.
			this.wwjPanel = new ApplicationTemplate.AppPanel(this.canvasSize, true);
			this.wwjPanel.setPreferredSize(canvasSize);
			ApplicationTemplate.insertBeforePlacenames(this.wwjPanel.getWwd(), layer1);
			ApplicationTemplate.insertBeforePlacenames(this.wwjPanel.getWwd(), layer2);

			setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			setVisible(true);

			//Call function to create JCheckBox on side window
			//JPanel sensorPanel = sensePanel();
			//JPanel larpanel = larPanel();

			JPanel selectPanel = new JPanel(new GridLayout(2,1));
			selectPanel.add(sensePanel());
			selectPanel.add(larPanel());

			JPanel selectPanel2 = new JPanel(new BorderLayout());
			selectPanel2.add(selectPanel, BorderLayout.NORTH);


			//selectPanel.setPreferredSize(this.selectPanelSize);

			this.getContentPane().add(selectPanel2, BorderLayout.WEST);
			this.getContentPane().add(wwjPanel, BorderLayout.CENTER);
			//this.getContentPane().add(controlPanel, BorderLayout.WEST);
			this.pack();

			// Center the application on the screen.
			Dimension prefSize = this.getPreferredSize();
			Dimension parentSize;
			java.awt.Point parentLocation = new java.awt.Point(0, 0);
			parentSize = Toolkit.getDefaultToolkit().getScreenSize();
			int x = parentLocation.x + (parentSize.width - prefSize.width) / 2;
			int y = parentLocation.y + (parentSize.height - prefSize.height) / 2;
			this.setLocation(x, y);
			this.setResizable(true);

			//yellow
			color = new Color(1f, 1f, 0f);
			//red
			Bcolor = new Color(1f, 0f, 0f);
			alpha = 1/10f;

			rdrShape = (AbstractSurfaceShape) radar;
			rdrShape.getAttributes().setDrawOutline(true);
			rdrShape.getAttributes().setOutlineMaterial(new Material(Bcolor));
			rdrShape.getAttributes().setOutlineOpacity(alpha);

			rdrShape.getAttributes().setInteriorMaterial(new Material(color));
			rdrShape.getAttributes().setInteriorOpacity(alpha);
			rdrShape.getAttributes().setDrawInterior(true);

			//green
			color = new Color(.196f,.99f,.196f);
			alpha = 5/10f;

			EOShape = (AbstractSurfaceShape) EO;
			EOShape.getAttributes().setDrawOutline(false);

			EOShape.getAttributes().setInteriorMaterial(new Material(color));
			EOShape.getAttributes().setInteriorOpacity(alpha);
			EOShape.getAttributes().setDrawInterior(true);

			this.layer1.addRenderable(this.radar);
			this.layer2.addRenderable(this.EO);

		}
		public void update(double lat, double lon, double heading, double EOHead, double EOfov, double eoSensorHole, double eoSensorRim, double radarRim, double radarHole)
		{

			//Update the radar sensor footprints location, heading, and altitude

			SurfaceSensor radarUpdate = (SurfaceSensor) radar;
			radarUpdate.setCenter(LatLon.fromDegrees(lat, lon));
			radarUpdate.setHeading(Angle.fromDegrees(heading+270));
			radarUpdate.setMajorRadius(radarRim);
			radarUpdate.setMinorRadius(radarRim);
			radarUpdate.setHoleRadius(radarHole);
			radarUpdate.setCircleAngle(Angle.fromDegrees(350));

			//Update the radar sensor footprints location, heaing, and altitude

			SurfaceSensor EOUpdate = (SurfaceSensor) EO;
			EOUpdate.setCenter(LatLon.fromDegrees(lat, lon));
			EOUpdate.setHeading(Angle.fromRadians(EOHead));
			EOUpdate.setCircleAngle(Angle.fromRadians(EOfov));

			EOUpdate.setHoleRadius(eoSensorHole);

			EOUpdate.setMinorRadius(eoSensorRim);
			EOUpdate.setMajorRadius(eoSensorRim);

			this.wwjPanel.getWwd().redraw();

		}

		//Syncs with frame
		protected void updateFrame(final double lat, final double lon, final double heading, final double eoHead, final double eofov)
		{

			// Ensure this is executed on the EDT.
			if (!SwingUtilities.isEventDispatchThread())
			{
				SwingUtilities.invokeLater(new Runnable()
				{
					public void run()
					{
						updateFrame(lat, lon, heading, eoHead, eofov);
					}
				});
			}
			else
			{
				update(lat, lon, heading, eoHead, eofov, this.minorRadius, this.majorRadius, this.radarRim,this.radarHole);
			}
        }

        //Calculates the neccesary inputs for each sensor
        public void doMath(double lat, double lon, double heading, double altitude, double EOHead, double depression, double EOfov)
        {

			//Convert to radians
			EOHead = EOHead*Math.PI/180;
			depression = depression*Math.PI/180;
			EOfov = EOfov*Math.PI/180;

			//Area of radar coverage
			this.radarRim = 1.23*Math.sqrt(altitude); //Radar horizon in nm
			this.radarRim = this.radarRim * 1852; // convert natical miles to meters

			//radarRim = 30000;
			//d = this.R*Math.acos(this.R/(this.R+altitude));

			this.radarHole=20000;

			//Check to make sure depression isnt too small
			depression = (depression <= Math.max(.5*EOfov,15*(Math.PI/180))) ? Math.max(.5*EOfov,15) : depression;
			//Check to make sure depression isnt too large
			depression = (depression > Math.PI) ? Math.PI : depression;

			if (depression <= Math.PI/2 -.5*EOfov)
			{
				//Sensor hole
				this.angle = Math.PI/2 - (depression+.5*EOfov);
				this.minorRadius = Math.tan(this.angle)*altitude;

				//MajorRadius
				this.angle = Math.PI/2 - (depression-.5*EOfov);
				this.majorRadius = Math.tan(this.angle)*altitude;
				//System.out.println("angle " + angle + "majorRadius"+majorRadius);
			}
			else if (depression == Math.PI)
			{
				this.minorRadius = 1;
				this.majorRadius = Math.tan(.5*EOfov)*altitude;
			}
			else
			{
				this.minorRadius = 1;
				this.majorRadius = 5000;
			}

			updateFrame(lat, lon, heading, EOHead, EOfov);
		}

		public void deleteRenderableRadar(Renderable shape)
		{
			this.layer1.removeRenderable(shape);
		}

		public void plusRenderableRadar(Renderable shape)
		{
			this.layer1.addRenderable(shape);
		}

		public void deleteRenderableEO(Renderable shape)
		{
			this.layer2.removeRenderable(shape);
		}

		public void plusRenderableEO(Renderable shape)
		{
			this.layer2.addRenderable(shape);
		}

		//Create sensor Panel button
		public JPanel sensePanel()
		{
			final Renderable radar = this.radar;
			final Renderable EO = this.EO;

			//rows, columns, hgap, vgap
			GridLayout layout = new GridLayout(1,2,20,0);
			JPanel ssPanel = new JPanel(layout);

			ssPanel.setBorder(BorderFactory.createTitledBorder("Sensor selection"));

			final JCheckBox b = new JCheckBox("Radar",true);

			ssPanel.add(b);
			final JCheckBox c = new JCheckBox("EO",true);

			ssPanel.add(c);

			b.addActionListener(new ActionListener()
			{
				boolean selected;

				public void actionPerformed(ActionEvent actionEvent)
				{

					if (b.isSelected()==false)
					{
						deleteRenderableRadar(radar);
					}
					else
					{
						plusRenderableRadar(radar);
					}

				}
			});

			c.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent actionEvent)
				{
					if (c.isSelected()==false)
					{
						deleteRenderableEO(EO);
					}
					else
					{
						plusRenderableEO(EO);
					}

				}
			});
			//ssPanel.setBorder(this.createTitleBorder("Sensors"));

			return ssPanel;
		}




	public JPanel larPanel()
	{

		GridLayout layout = new GridLayout(1,1,20,0);

		JPanel LARpanel = new JPanel(layout);

		LARpanel.setBorder(BorderFactory.createTitledBorder("LAR overlay"));

		final JCheckBox a = new JCheckBox("LAR",true);

		LARpanel.add(a);

		a.addActionListener(new ActionListener()
		{
			boolean selected;

			public void actionPerformed(ActionEvent actionEvent)
			{

				if (a.isSelected()==false)
					{
						//deleteRenderable(radar);
					}
					else
					{
						//plusRenderable(radar);
					}

			}
		});

		return LARpanel;
	}

}

	public static void main(String[] args)
	{
		int end = 0;
		double lat;
		double lon;
		try
		{

		readfile start = new readfile();
		start.Read();
		}
		catch (Exception e)
		{
		e.printStackTrace();
		}


	}

}